@@ -14,7 +14,7 @@ import subprocess
 import tempfile
 import pickle
 from . import infin_ast
-from datetime import datetime
+from datetime import datetime, timedelta
 import time
 from mlflow import start_run, end_run, log_metric, log_param, log_artifacts, set_experiment
 from os.path import sep
@@ -142,7 +142,15 @@ def s3downloader(download_task_q, write_pipe, endpoint, glb):
     client = session.client('s3', endpoint_url=endpoint)
     while (True):
         try:
+            if (verbose == True):
+                start = datetime.now()
             download_task = download_task_q.get()
+            if (verbose == True):
+                end = datetime.now()
+                td = end - start
+                ms = (td.days * 86400000) + (td.seconds * 1000) + (td.microseconds / 1000)
+                if (ms > 10):
+                    info('s3downloader: q.get took ' + str(ms) + ' ms')
         except Empty as e:
             info('s3downloader: No more entries in download_task_q. Exiting..')
             write_pipe.close()
@@ -156,16 +164,36 @@ def s3downloader(download_task_q, write_pipe, endpoint, glb):
                 full_object_key = filename
             else:
                 full_object_key = parentdir + sep + filename
-            # info('s3downloader: starting download of ' + full_object_key + ' from ' + bucketname)
+            if (verbose == True):
+                info('s3downloader: starting download of ' + full_object_key\
+                        + ' from ' + bucketname)
+                start = datetime.now()
             obj = client.get_object(Bucket=bucketname, Key=full_object_key)
-            # info('s3downloader: finished download of ' + full_object_key + ' from ' + bucketname)
+            content_length = obj['ContentLength']
+            if (verbose == True):
+                end = datetime.now()
+                info('s3downloader: finished download of ' + full_object_key\
+                        + ' from ' + bucketname)
+                td = end - start
+                ms = (td.days * 86400000) + (td.seconds * 1000) + (td.microseconds / 1000)
+                if (ms > 500):
+                    info('s3downloader: download of size ' + str(content_length) + ' took '\
+                        + str(ms) + ' ms')
+
             strbody = obj['Body']
-            datetime = obj['LastModified']
+            dtm = obj['LastModified']
             bts = strbody.read()
-            key = datetime.strftime('%Y-%m-%d %H:%M:%S') + ' ' + bucketname + '/' + full_object_key
+            key = dtm.strftime('%Y-%m-%d %H:%M:%S') + ' ' + bucketname + '/' + full_object_key
             if (glb == None):
-                write_pipe.send(key)
-                write_pipe.send(bts)
+                if (not write_pipe.writable):
+                    info('s3downloader: WARNING write pipe not writable')
+                    time.sleep(1)
+                try:
+                    write_pipe.send(key)
+                    write_pipe.send(bts)
+                except Exception as e:
+                    status_str = str(e)
+                    info("Error sending bytes back: " + status_str)
             else:
                 try:
                     namespaced_infin_transform_one_object(bucketname, parentdir, filename, bts)
@@ -177,7 +205,7 @@ def s3downloader(download_task_q, write_pipe, endpoint, glb):
                 write_pipe.send(key)
                 write_pipe.send(status_str)
         elif (op == 'quit'):
-            # info('s3downloader: Received quit')
+            info('s3downloader: Received quit')
             write_pipe.close()
             break
         else:
@@ -250,14 +278,10 @@ def download_objects_inner(endpoint, bucketname, parentdir, array_of_files, is_c
     with tqdm(total=len(array_of_files)) as pbar:
         files_read = 0
         while pipe_from_child:
-            ready = wait(pipe_from_child, timeout=2)
-            if (ready == None):
-                info('>>>>>>>>>>>>>> received None from wait >>>>>>>>>>>>')
-            else:
-                if (len(ready) == 0):
-                    info('>>>>>>>>>>>>>> WARNING: None ready from wait >>>>>>>>>>>>')
-                for read_pipe in ready:
-                    try:
+            ready = wait(pipe_from_child, timeout=10)
+            for read_pipe in pipe_from_child:
+                try:
+                    while (read_pipe.poll()):
                         key = read_pipe.recv()
                         bts = read_pipe.recv()
                         filekeys.append(key)
@@ -266,10 +290,9 @@ def download_objects_inner(endpoint, bucketname, parentdir, array_of_files, is_c
                         files_read += 1
                         if ((files_read % 10) == 0):
                             log_metric("downloaded", files_read, step=step)
-                            info("Downloaded " + str(files_read))
                             step += 1
-                    except EOFError:
-                        pipe_from_child.remove(read_pipe)
+                except EOFError:
+                    pipe_from_child.remove(read_pipe)
 
     for i in range(num_threads()):
         processes[i].join()
